#ifndef _Externals_h
#define _Externals_h 1

// projection of 62 terms to 13 input terms
// using this is simpler as using the old term62[13][num[13]] method
const int term_13[63]={   /* element 0 is not used */
  0, 1,1,1,1,1, 2,2, 3,3,3,3,3,3, 4,4, 5,5,5,5,5,5, 6,6, 7,7, 8,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9, 3,3,3, 5,5,5, 6, 7, 10,
  11,11,11, 12,12,12, 13};
      
struct prm {

  /* This structure contains all of the globally visible information
     concerning runtime CCM Parameters set by the users either in the
     code itself, as command line instructions, using the GUI, or in
     some input file. 
     
     This is distinct from Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
     
  */


  int n;                // SUBn-m (n) 
  int m;                // SUBn-m (m) 
  int btLength;         // Joerg Oct05: length of bounding triangle? == m
  int dim;              // Dimension Of Lattice
  int nFundG;           // # Fund. Configs G-State
  int nFundE;           // # Fund. Configs E-State
  // float s;              // Global Quantum Spin Number (removed Joerg Sep05)
  int s2;               // twofold of Global Quantum Spin Number
  int ddd;
  int bonds;            // number of `bonds'
  
};
extern prm Params;

extern int  max_number_cells_BT;

struct ctrl {
  /* 
     
     Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
     
  */

  int output;           // Level Of Output
  int file;             // save CCM equations to file
  char name[200];        // Root File Name                

};


extern ctrl Control;

struct UC {

  /* this structure completely defines the unit cell */

  int number;             // number of sites in unit cell (UC)
  double x[MAX_UNIT_CELL];   // number of sites in UC
  double y[MAX_UNIT_CELL];   // number of sites in 
  double z[MAX_UNIT_CELL];   // number of sites in 
  // float s[MAX_UNIT_CELL]; // `local' quantum spin number for each site in UC
  int  s2[MAX_UNIT_CELL]; // 2x `local' quantum spin number for each site in UC

};

extern UC UnitCell;

struct sub {

  /* which terms from the unit cell to determine the
     sublattice for */

  int number;
  int cell[MAX_UNIT_CELL];

};

extern sub sublatticeMagnetisation;

struct neighbours {

  /* this is an interesting structure because it defines the 
     nearest neighbour structure for each of the elements of 
     the unit cell.

     Essentially each unit cell has however many nearest-neighbours,
     some of which in within the unit cell itself, and some which
     are in neigbouring cells.

     This information is most easily coded in by the user, or
     by writing a simple bit of code to determine the neighbours
     matrix of the lattice or Hamiltonian.

     Note that this also offers the possiblity of defining
     nearest-neighbours, next-nearest-neighbours, next-next-...
     and so on.

  */

  int number[MAX_UNIT_CELL]; /* number of nearest-neighbours for
				each site in the UC */
  int x[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION x bravais
  int y[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION y bravais
  int z[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION z bravais
  int cell[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // which site in UC
  
};
extern neighbours LatticeNeighbours;

struct bravais {

  /* structure defines the bavais lattice */

  double a[4];
  double b[4];
  double c[4];

};

extern bravais BravaisLattice;

struct symmetries {

  /* this structure contains all of the information concerning
     the symmetries of the lattice, Hamiltonian, unit cell etc. */

  int number;     // number of symmetries in lattice, Hamiltonian etc

  int switchFunction(int switchVariable, 
		     double x1, 
		     double y1, 
		     double z1,
		     double &x2, 
		     double &y2,
		     double &z2);

  double R[MAX_NUM_SYM][4][4]; // ROTATION MATRIX

  double x[MAX_NUM_SYM];       // NON-BRAVAIS LATTICE VECTOR.
  double y[MAX_NUM_SYM];
  double z[MAX_NUM_SYM];

};

extern symmetries LatticeSymmetries;

struct spinConfigs {
  
  /* 

     This structure describes a particular spin configuration by storing 
     the information in an array. The size of the (integer) numbers
     becomes too long for saving as a standard `int' -- or even `long int.'
     
     Each integer element gives the information on 20 lattices sites
     each of which has a spin of quantum spin number Params.s or
     UnitCell.s[]
     
  */
  
  int descriptor[MAX_SPIN_LIST+3];
  
};

extern spinConfigs spinConfigurations[MAX_NUM_CONFIGURATIONS];
extern spinConfigs latticeAnimals[MAX_NUM_CONFIGURATIONS];
// extern spinConfigs connectedConfigurations[MAX_NUM_CONFIGURATIONS];
extern int         NumberConfigurations;
extern int         NumberLatticeAnimals;

extern double CCMcoeffs[MAX_NUM_CONFIGURATIONS];


#define MAX_TERMS_h  63   // was 200 (Sep05 Joerg)
#define MAX_TERMS_t  17   // was 17, partly 200
#define MAX_BONDS     3   // 2 was the default until Sep05, Joerg

//     terms used range=typeArray[1..13][l]=1..62 (see input.C)
struct hamiltonian_defn {

  int evaluate;
  double coefficient[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; 

  int k_to_m_number[MAX_UNIT_CELL];
  int k_to_m[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][4];  // [x,y,z,cell] 

  int m_to_k_number[MAX_UNIT_CELL];                
  int m_to_k[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][4];  // [x,y,z,cell]
};

//  array index term moved to simplify code (Joerg Sep05)
// read in from data-file
extern hamiltonian_defn Hamiltonian_types[MAX_BONDS+1][MAX_TERMS_t];
//  similarity transformed version of Hamiltonian_type (see input.C)
extern hamiltonian_defn Hamiltonian      [MAX_BONDS+1][MAX_TERMS_h];

// other global variables

extern int lookUpTable[MAX_BT+1][5];
extern int BTarray[MAX_LSUBM+1][MAX_LSUBM+1][MAX_LSUBM+1];

extern int    totalPoints;

/* obtain */

extern int binary_max[31];
extern int trimary_max[MAX_LSUBM+1];
extern int quartary_max[MAX_LSUBM+1];

// files

extern char terms_fname[80]; /* term file fileName */
extern int  terms_shm_fd;    /* term file descriptor, if SHM used */
extern FILE *terms_fp;       /* term file */
extern FILE *fp_gs;   /* gs-terms */
extern FILE *fp1;     /* input.ccm */
extern FILE *fp2;     /* output.results */
/*
 * - linux has no problems by using all free memory as IO cache
 *   and give it back on demand, so we dont need lot of extra buffer
 * - solaris has a buffer cache of fixed size, this is bad and
 *   can cause excessiv disk access where there is enough free memory 
 *   or dont give back memory when other jobs need it
 * - so the user can choose full buffering to reduce disk access to 0
 *   
 */
extern char *fp_buf; /* read/write buffer may be bigger than the file */
        
// solve variables.

extern double maximum;
extern double minimum;
extern double increment;

// restriction on choice of clusters.

extern int restrictionClusters;

// direction bonds or not?

extern int directional;

// restriction of sweeping for F_k F_m s_k^z s_m^z terms.

extern int midPointX;
extern int midPointY;
extern int midPointZ;
extern int radius;

// solution strategy

extern int solutionStrategy;

// parallel variables

extern int myid;
extern int npe;
#endif
